@@ -283,10 +283,7 @@ fit_model <- function(model, backend, ...) {
   )
   # transform into stanfit object for consistent output structure
   out <- rstan::read_stan_csv(out$output_files())
-  # read_stan_csv seems to accidentally rename dimension suffixes
-  for (i in seq_along(out@sim$samples)) {
-    names(out@sim$samples[[i]]) <- out@sim$fnames_oi
-  }
+  out <- repair_stanfit_names(out)
   # allow updating the model without recompilation
   attributes(out)$CmdStanModel <- model
   attributes(out)$metadata <- metadata
@@ -528,6 +525,19 @@ validate_silent <- function(silent) {
   silent
 }
 
+# repairs parameter names of stanfit objects
+repair_stanfit_names <- function(x) {
+  stopifnot(is.stanfit(x))
+  # the posterior package cannot deal with non-unique parameter names
+  # this case happens rarely but might happen when sample_prior = "yes"
+  x@sim$fnames_oi <- make.unique(x@sim$fnames_oi, "__")
+  for (i in seq_along(x@sim$samples)) {
+    # rstan::read_stan_csv may have renamed dimension suffixes (#1218)
+    names(x@sim$samples[[i]]) <- x@sim$fnames_oi
+  }
+  x
+}
+
 # possible options for argument 'file_refit'
 file_refit_options <- function() {
   c("never", "always", "on_change")
